$TITLE (iDCM 44 COMMUNICATIONS FIRMWARE: SERIAL NON-AUTO MODULE)
$DATE (8 AUGUST 86)
$NOMO
$PAGING
$REGISTERBANK (0)

;***************************************************************************
;*
;*     TITLE:          iDCM 44 COMMUNICATIONS FIRMWARE: SERIAL NON-AUTO MODULE
;*
;*     RELEASE:        2.0
;*
;*     DESCRIPTION:    THIS MODULE CONTAINS THE SERIAL PROCEDURES
;*
;*     UPDATES:        

;*
;***************************************************************************

;*         

;*           COPYRIGHT 1983, 1986 INTEL CORPORATION

;*

;*           INTEL CORPORATION PROPRIETARY INFORMATION.  THIS LISTING

;*           IS SUPPLIED UNDER THE TERMS OF A LICENSE  AGREEMENT WITH

;*           INTEL CORPORATION  AND MAY NOT  BE COPIED NOR  DISCLOSED

;*           EXCEPT IN  ACCORDANCE WITH THE  TERMS OF THAT AGREEMENT.

;* 

;***************************************************************************

NAME    DCM44N



USING 0


$nolist
$include(reg44.pdf)
$include(dcm44c.ext)
$include(dcm44c.lit)
$include(r2sys.mac)
$include(dcm44m.a51)

$list


public  non_auto_slave_rcv, non_auto_master_rcv, set_slave_offline

public  serial_timeout, swap_station_states

extrn   bit (s_station_mode, s_timeout_count, s_timeout_set)

extrn   bit (s_init_mode, s_info_mode, s_poll_mode, s_set_rts)

extrn   data (s_queue_ptr, s_poll_queue_in, s_poll_queue_out)

extrn   data (s_last_nsnr, buf_size, req_next_buffer)


extrn   xdata (s_station_state, s_station_poll)


extrn   code (event_detect, serial_gateway_switch, check_receive_buffer)

extrn   code (initiate_order_transmission, auto_send_acknowledge_check)

DCMFW_CODE_SEG          SEGMENT CODE
DCMFW_BIT_SEG           SEGMENT DATA    BITADDRESSABLE
DCMFW_INT_SEG           SEGMENT DATA
DCMFW_EXT_SEG           SEGMENT XDATA
DCMFW_STATION_SEG       SEGMENT XDATA   INPAGE
DCMFW_POLL_SEG          SEGMENT XDATA   INPAGE

RSEG                    DCMFW_CODE_SEG


$eject
;***************************************************************************
;*
;*     ENTRY:          non_auto_slave_rcv
;*
;*     PURPOSE:        Process a frame received by a slave node in non-auto

;*                     mode
;*
;*     ENTERED FROM:   serial_interrupt_check
;*
;*     CALLS:          check_receive_buffer
;*
;*     ENTERS:         auto_send_acknowledge_check, event_detect,

;*                     set_non_auto_transmit
;*
;*     INPUTS:         none
;*
;*     RETURNS:        nothing
;*
;*     DESCRIPTION:    If a sequence error occurred, then go to the

;*                       slave protocol error routine.
;*                     Next, the type of received frame is checked. 
;*                     If it is an information or supervisory frame, then

;*                       either buffer-overflow occurred, or a frame was

;*                       received before the slave was able to get into auto

;*                       mode (after initialization was complete).  In any

;*                       case, go back into auto mode, reenable the receiver,

;*                       and go make an i-frame acknowledge check with the

;*                       rts-flag set, indicating that the rts siu bit needs

;*                       to be set. 

;*                     If it is an unnumbered frame, then if the slave is in

;*                       init mode, then jump to slave_init_generate, else

;*                       jump to slave_protocol_error.

;*
;***************************************************************************

non_auto_slave_rcv:

; A frame was received while the slave was in non-auto mode.  

; First check for type of frame.

;

    jb      ses, slave_protocol_error  ;if protocol error, go process

    jb      ser, slave_protocol_error  ;

    mov     a, rcb                     ;determine frame type

    jnb     acc.i_frame_bpos, slave_info_rcv ;if info frame, jump

    jnb     acc.s_frame_bpos, slave_supv_rcv ;if supervisory frame, jump

    jnb     s_init_mode, slave_protocol_error ;if u-frame, no init mode, jump

    sjmp    slave_init_generate        ;if u-frame, init mode, jump



slave_info_rcv:

slave_supv_rcv:

; An information or supervisory frame was received either before the slave

; was able to go into auto mode, or after the slave was in auto mode but 

; was kicked out of auto mode.  These conditions can be caused by buffer 

; overflow, or receiving the first I-frame before the slave is able to 

; go into auto mode (as the last step in synchronization).

;

    acall   check_receive_buffer       ;check on receive buffer

    setb    am                         ;set slave back into auto mode

    setb    s_set_rts                  ;set rts flag

    setb    rbe                        ;reenable receiver

    ajmp    auto_send_acknowledge_check ;go check for msg. ack.



;---------------------------------------------------------------------------
;  Entry Point:    slave_init_generate

;
;  Purpose:        Determine next control frame for slave to transmit
;                  during synchronization, and transmit the frame.

;

;  Entered From:   non_auto_slave_rcv, protocol_error_reset
;
;  Calls:          check_receive_buffer
;
;  Enters:         set_non_auto_transmit, which enters event_detect
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    Checks last transmit control byte against current
;                  received control byte for a slave station. For trans-
;                  mitted anything and received disc, it transmits a ua.
;                  For transmitted ua and received snrm, it transmits a ua,
;                  clears sequence numbers, and later (with transmit ack)
;                  goes into auto mode.  For anything else, it transmits
;                  a frmr.
;---------------------------------------------------------------------------

slave_init_generate:

    setb    s_init_mode                     ;reset flags
    clr     s_info_mode                     ;
    mov     a,rcb                           ;identify the received frame
    cjne    a,#disc_msk,slave_snrm_chk      ;if not disc, jump

    mov     tcb,#ua_msk                     ;else xmit ua in response
    ajmp    set_non_auto_transmit           ;go xmit unnumbered frame


slave_snrm_chk:
    cjne    a,#snrm_msk,slave_other         ;if not snrm, jump

    mov     a,#ua_msk                       ;was a ua xmitted prev to snrm?

    cjne    a,tcb,slave_other               ;if not, xmit frmr
;   mov     tcb,#ua_msk                     ;else transmit ua
    mov     nsnr,#0                         ;initialize ns and nr counts
    clr     s_init_mode                     ;turn off init mode
    mov     sts,#10100000b                  ;tbf=1,rts=1,rest=0

    acall   check_receive_buffer            ;Check for receive buffer

    jnb     si,$                            ;wait for xmit complete interrupt

    mov     sts,#01000010b                  ;rbe=1,am=1,rest=0

    ajmp    event_detect                    ;go wait on event


slave_other:

; if not snrm, then it is either an error (0) or an unrecognized frame.

; generate a frame reject to force resynchronization.

;
    mov     tcb,#frmr_msk                   ;set for transmitting frmr
    ajmp    set_non_auto_transmit           ;go xmit unnumbered frame


;---------------------------------------------------------------------------
;  Entry Point:    slave_protocol_error 

;
;  Purpose:        Handle protocol error conditons on a slave node

;

;  Entered From:   non_auto_slave_rcv 
;
;  Calls:          serial_gateway_switch
;
;  Enters:         slave_init_generate
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    This routine processes a non-recoverable error on the  

;                  slave node by returning all message queued for transmission

;                  to the System Buffer Pool, clearing the siu transmit    

;                  registers, and clearing the info mode flag.  It then jumps

;                  to slave_init_generate in order to transmit a disc frame

;                  back to the master.

;---------------------------------------------------------------------------


slave_protocol_error:   

    mov     tbs,#0                          ;clear siu xmit registers

    mov     tbl,#0                          ;

slave_recycle_msgs:                         ;empty all msgs. from queue

    %dequeue_message_def(s_queue_ptr)       ;is there another message?

    jz      slave_recycle_end               ;if not, exit loop

    %req_deallocate                         ;else, deallocate next msg.

    sjmp    slave_recycle_msgs              ;check for another msg.

slave_recycle_end:

    ajmp    slave_init_generate             ;go xmit a snrm to the master



$eject    

;***************************************************************************
;*
;*     ENTRY:          non_auto_master_rcv
;*
;*     PURPOSE:        Process a frame received by a master node.

;*
;*     ENTERED FROM:   serial_interrupt_check
;*
;*     CALLS:          serial_gateway_switch, check_receive_buffer

;*                     enqueue_poll_station_def (macro),

;*                     dequeue_poll_station_def (macro),

;*                     swap_station_states_def  (macro),

;*                     dequeue_message_def      (macro),

;*                     req_deallocate           (macro)

;*
;*     ENTERS:         event_detect, initiate_order_transmission

;*                     
;*
;*     INPUTS:         none
;*
;*     RETURNS:        nothing
;*
;*     DESCRIPTION:    If a sequence error occurred, then go to the

;*                       master protocol error routine.
;*                     Next, the type of received frame is checked. 
;*                     If it is an information frame, then jump to the

;*                       master info receive routine.

;*                     If it is a supervisory frame, then go the the master

;*                       ack check routine to process any acknowledgement.

;*                     If it is an unnumbered frame, then if the master is in

;*                       init mode, then jump to master_init_generate, else

;*                       jump to master_protocol_error.

;*
;***************************************************************************

non_auto_master_rcv: 
    jb      ses, master_protocol_error      ;if protocol error, jump

    jb      ser, master_protocol_error      ;

    mov     a, rcb                          ;determine frame type

    jnb     acc.i_frame_bpos, master_info_rcv ;if info frame, jump

    jnb     acc.s_frame_bpos, master_ack_check ;if supervisory frame, jump

    jnb     s_init_mode, master_protocol_error ;if u-frame, non-init, jump

;   ajmp    master_init_generate            ;if u-frame, init-mode, jump



;---------------------------------------------------------------------------
;  Entry Point:    master_init_generate

;
;  Purpose:        Determine next control frame for master to transmit
;                  during synchronization, and transmit the frame.

;

;  Entered From:   non_auto_master_rcv, protocol_error_reset
;
;  Calls:          none
;
;  Enters:         set_non_auto_transmit, which enters event_detect
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    Checks last transmit control byte against current
;                  received control byte for a master station. For trans-
;                  mitted disc and received ua, it generates a snrm.  For
;                  transmitted snrm and received ua, it clears the sequence
;                  numbers and exits init mode.  Any other transmitted control
;                  field transmits a disc.

;---------------------------------------------------------------------------


master_init_generate:
    setb    s_init_mode                     ;reset flags
    clr     s_info_mode                     ;
    clr     s_poll_mode                     ;
    mov     a,rcb                           ;identify the received frame

    cjne    a,#ua_msk,master_disc_xmit      ;if not a ua, go xmit disc

    mov     a,#disc_msk                     ;see if a disc was xmitted
    cjne    a,tcb,master_snrm_chk           ;if not, go check for snrm      
    mov     tcb,#snrm_msk                   ;if disc xmitted before, xmit snrm
    ajmp    set_non_auto_transmit           ;go xmit unnumbered frame


master_disc_xmit:
    mov     tcb,#disc_msk                   ;transmit a disc

    ajmp    set_non_auto_transmit           ;go xmit unnumbered frame

master_snrm_chk:        
    mov     a,#snrm_msk                     ;see if a snrm was xmitted

    cjne    a,tcb,master_disc_xmit          ;if not, go xmit disc
    mov     nsnr,#0                         ;zero the sequence numbers

    clr     s_init_mode                     ;take out of init mode

    ajmp    sps_transmit_rr                 ;and go transmit rr


;---------------------------------------------------------------------------
;  Entry Point:    master_protocol_error

;
;  Purpose:        Handle protocol error conditons

;

;  Entered From:   non_auto_master_rcv
;
;  Calls:          serial_gateway_switch
;
;  Enters:         station_swap_check
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    First checks the context in which the error occurred.  

;                  If a message was being transmitted (info mode), or the 

;                  master was synchronizing with the slave (init mode) in order

;                  to transmit a message, then the top message on the message

;                  queue is returned to the sender, and the slave is set 

;                  offline.  Otherwise, it is assumed that the protocol error

;                  occurred while polling for a reply message, and the slave

;                  is set offline. 

;---------------------------------------------------------------------------


master_protocol_error:

    clr     s_poll_mode                     ;clear poll-mode flag regardless


;if in init mode, return the message that caused synchronization attempt

    jbc     s_init_mode,master_abort_info_send



;if in info mode, return the message to the task

    jbc     s_info_mode,master_abort_info_send 



;else, assume that protocol error occurred during a poll send

    sjmp    master_abort_slave              ;


master_abort_info_send:
    %dequeue_message_def(s_queue_ptr)       ;dequeue message

    jz      master_abort_slave              ;if none on queue, jump

    mov     tbs,#0                          ;clear siu xmit registers
    mov     tbl,#0                          ;

    %set_msg_ptr(message_route_ofs)         ;change msg. type to reply
    mov     a,@r1                           ;

    setb    acc.msg_type_bpos               ;

    mov     @r1,a                           ;

    %set_msg_ptr(message_reply_ofs)         ;indicate error in reply field

    mov     @r1,#e_protocol_error           ;

    acall   serial_gateway_switch           ;return message to sender



master_abort_slave:

    mov     a,stad                          ;set station state to offline

    acall   set_slave_offline               ;

    ajmp    station_swap_check              ;go service next slave station



;---------------------------------------------------------------------------
;  Entry Point:    master_info_rcv

;
;  Purpose:        Handle incoming information frames to master station.    

;

;  Entered From:   non_auto_master_rcv
;
;  Calls:          serial_gateway_switch
;
;  Enters:         master_ack_check
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    First checks for buffer overflow.  If overflow, then it

;                  is assumed that another buffer will be allocated in

;                  master_ack_check, and that process is called.  Otherwise,

;                  the received message is routed through the gateway, and

;                  the receive registers cleared.  If no more replies are

;                  expected from the secondary station (nr=ns) then the

;                  station is removed from the poll list.  Finally, the

;                  frame is checked so see if it also acknowledges a 

;                  previous send.    

;---------------------------------------------------------------------------


master_info_rcv:

    jb      bov,master_ack_check            ;if bov, ignore frame & check

                                            ;for new buffer

    mov     r0,rbs                          ;else set the message pointer
    dec     r0                              ;

    dec     r0                              ;
    acall   serial_gateway_switch           ;and return the msg. to its sender
    mov     rbs,#0                          ;clear the receiver registers

    mov     rbl,#0                          ;

;   sjmp    master_ack_check                ;see if prev. send acknowledged


master_ack_check:

    acall   check_receive_buffer            ;check on receive buffer

    mov     a,tbs                           ;was a message transmitted?

    jz      station_swap_check              ;if not, jump


master_info_xmitted: 
    mov     a,s_last_nsnr                   ;see if ns field of xmitted I

    xrl     a,nsnr                          ;control byte is different (one

    anl     a,#nsnr_ns_msk                  ;greater) than ns field of nsnr

    jz      set_non_auto_transmit           ;if not, retransmit message


message_acknowledged:

    clr     s_info_mode                     ;clear info mode flag

    mov     tbs,#0                          ;clear transmit registers

    mov     tbl,#0                          ;

    %dequeue_message_def(s_queue_ptr)       ;get msg. off of xmit queue

    %req_deallocate                         ;deallocate xmitted msg. buffer

    acall   check_receive_buffer            ;see if receiver needs a buffer
;   ajmp    station_swap_check              ;and go service next station

;---------------------------------------------------------------------------
;  Entry Point:    station_swap_check

;
;  Purpose:        master checks for a new station to send a message to or

;                  poll.

;

;  Entered From:   master_ack_check    
;
;  Calls:          initiate_order_transmission 

;                  enqueue_poll_station_def (macro)

;                  dequeue_poll_station_def (macro)
;
;  Enters:         event_detect
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    First checks to see if the present station should be    

;                  enqueued in the station poll queue.  It then looks at the 

;                  serial message queue for a message to transmit.  If so,  

;                  the destination slave is swapped in and the message is 

;                  transmitted.  If there are no messages to send, the master

;                  checks its polling queue for stations to poll for   

;                  outstanding replies.  If the queue is not empty, a slave

;                  is swapped in and polled.                         

;---------------------------------------------------------------------------



station_swap_check:
    mov     a,nsnr                          ;are there any frames outstanding

    swap    a                               ;on this slave (ie, is ns <> nr)?

    xrl     a,nsnr                          ;

    jz      ssc_next_station                ;if not, jump
    %enqueue_poll_station_def               ;else put this station on the
                                            ;poll queue 
ssc_next_station:

    clr     s_poll_mode                     ;clear poll-mode flag
    mov     a,s_queue_ptr                   ;check for msg. to be xmitted
    jnz     swap_and_xmit                   ;if one, swap in station & xmit
    %dequeue_poll_station_def               ;else check for station to be polled
    jnz     station_poll_swap               ;if there is one, jump

    setb    rbe                             ;and go wait on the
    ajmp    event_detect                    ;next event



swap_and_xmit:                              ;transmit message dequeued

    mov     r0,a                            ;r0 => msg. to be xmitted

    acall   initiate_order_transmission     ;from serial message queue

    ajmp    event_detect                    ;then wait on next event

;---------------------------------------------------------------------------
;  Entry Point:    station_poll_swap

;
;  Purpose:        swap out current station, swap in new station, and transmit

;                  rr type of poll.

;

;  Entered From:   station_swap_check  
;
;  Calls:          swap_station_states_def (macro)

;
;  Enters:         event_detect
;
;  Inputs:         acc = station address
;
;  Returns:        none
;
;  Description:    Checks new station against current station.  If not the 

;                  same then nsnr is written to the old station's

;                  s_station_state, and the new station's s_station_state

;                  is written to nsnr.  In either case, the transmit control

;                  byte is set to an rr with the nr value taken from nsnr.   

;                  then the transmit is initiated.

;---------------------------------------------------------------------------


station_poll_swap:
    cjne    a,stad,sps_swap_out             ;if no station match, go swap out

    mov     a,smd                           ;see if in synchronous mode

    anl     a,#11100000b                    ;

    jnz     sps_transmit_rr                 ;if not, skip pause code

    mov     r6,#020h                        ;execute pause loop to prevent

    djnz    r6,$                            ;master from timing out
    

sps_transmit_rr:                            ;poll next station

    mov     a,nsnr                          ; set up nsnr registers

    mov     s_last_nsnr,a                   ; build control byte

    swap    a                               ;
    anl     a,#nr_msk                       ;
    orl     a,#rr_msk                       ;
    mov     tcb,a                           ;
    setb    s_poll_mode                     ;

;   ajmp    set_non_auto_transmit           ;



set_non_auto_transmit:                      ;initiate non-auto transmit
    mov     sts,#11100000b                  ; rts=1,tbf=1,rbe=1,rest=0

    ajmp    event_detect                    ;and go wait for xmit complete


sps_swap_out:
    acall   swap_station_states             ;swap in nsnr values 
    sjmp    sps_transmit_rr                 ;and go transmit poll rr


;---------------------------------------------------------------------------

; 

;      PROCEDURE:      swap_station_states

; 

;      PURPOSE:        swap station states saves the NSNR value for the

;                      current station and restores the NSNR for the next.

; 

;      CALLED BY:      station_pool_swap, initiate_order_transmission

; 

;      CALLS:          None

; 

;      INPUTS:         Station Address: ACC

; 

;      RETURNS:        Nothing

; 

;      DESCRIPTION:    swap_station_states puts the NSNR register value

;                      into the location of the S_Station_State array

;                      indexed by the value in the STAD register.  Then it

;                      gets the value stored in the location of the

;                      S_Station_State array indexed by the ACC and puts it

;                      in the NSNR register.  The value in the ACC is put

;                      into the STAD register.

; 

;---------------------------------------------------------------------------



swap_station_states:

    cjne    a,stad,swap_states              ;if new stad, swap

    ret                                     ;else do nothing

swap_states:

    mov     dptr, #s_station_state          ;save old station NSNR

    mov     dpl, stad

    xch     a, stad                         ;set new station address

    mov     a, nsnr

    movx    @dptr, a

    mov     dpl, stad                       ;set new station NSNR

    movx    a, @dptr

    mov     nsnr, a

    ret



;---------------------------------------------------------------------------
;  Entry Point:    serial_timeout

;
;  Purpose:        Handles response timeouts from secondary
;

;  Entered From:   event_detect
;
;  Calls:          none
;
;  Enters:         set_non_auto_transmit, which enters event_detectx

;                  protocol_error_reset
;
;  Inputs:         none
;
;  Returns:        none
;
;  Description:    checks timeout flag to see if this is the first or
;                  second time the timeout has occurred.  If it is the first,

;                  then the frame is retransmitted.  If it is the second, then 

;                  the frame is handled as described in protocol_error_reset.

;---------------------------------------------------------------------------

serial_timeout:
    jbc     s_timeout_count,terminate_station ;if 2nd timeout, branch

    setb    s_timeout_count                 ;else flag as a retransmit

    ajmp    set_non_auto_transmit           ;and retransmit the frame

        
terminate_station:                          ;this is the 2nd timeout
    clr     s_timeout_set                   ;therefore reset the timeout flag

    ajmp    master_protocol_error           ;and set station "offline"


;---------------------------------------------------------------------------
;  Procedure:      set_slave_offline

;
;  Purpose:        Set a slave station to the offline state.  This forces

;                  the master to resynchronize on the next message sent to 

;                  the slave.

;

;  Called From:    master_protocol_error, rac_offline
;
;  Calls:          none
;
;  Inputs:         acc = the address of the station to be set offline
;
;  Returns:        none
;
;  Description:    Sets the slaves entry in the s_station_state table to 

;                  the offline (0ffh).  If the station address is the present

;                  station being referenced (if acc = the siu stad register),  

;                  then the siu nsnr value is also set to offline (0ffh).

;---------------------------------------------------------------------------


set_slave_offline:

    mov     dptr,#s_station_state           ; 

    mov     dpl,a                           ;dptr => state entry

    cjne    a,stad,sso_pass                 ;if not present station, jump

    mov     nsnr,#ofl_msk                   ;else set nsnr to offline

sso_pass:
    mov     a,#ofl_msk                      ;set state table entry to offline
    movx    @dptr,a                         ;
    ret                                     ;return to caller

        
END
