$TITLE (iDCM 44 COMMUNICATIONS FIRMWARE: RAC TASK MODULE)

$DATE (8 AUGUST 86)

$NOMO

$PAGING

$REGISTERBANK (0)



;***************************************************************************

;*

;*     TITLE:          iDCM 44 COMMUNICATIONS FIRMWARE: RAC TASK MODULE

;*

;*     RELEASE:        2.0

;*                     2.1 -- Changed to coincide with iDCX R2.1.

;*

;*     DESCRIPTION:    THIS MODULE CONTAINS THE RAC TASK PROCEDURES

;*

;*     UPDATES:

;*

;***************************************************************************

;*         

;*           COPYRIGHT 1983, 1986 INTEL CORPORATION

;*

;*           INTEL CORPORATION PROPRIETARY INFORMATION.  THIS LISTING

;*           IS SUPPLIED UNDER THE TERMS OF A LICENSE  AGREEMENT WITH

;*           INTEL CORPORATION  AND MAY NOT  BE COPIED NOR  DISCLOSED

;*           EXCEPT IN  ACCORDANCE WITH THE  TERMS OF THAT AGREEMENT.

;* 

;***************************************************************************



NAME    DCM44R



USING 0



$nolist

$include(reg44.pdf)

$include(dcm44c.lit)

$include(r2sys.mac)

$include(dcm44m.a51)

$list



public  process_rac_cmd

        

extrn   code    (local_gateway_switch, reqcreatetask, reqdeletetask)

extrn   code    (reqgetfunctionids, reqreleasemem, rqsystemstart)

extrn   code    (set_slave_offline)



extrn   data    ( rqbufsize )



dcmfw_code_seg          segment code

dcmfw_bit_seg           segment data    bitaddressable

dcmfw_ext_seg           segment xdata



rseg                    dcmfw_bit_seg



extrn   bit     ( s_station_mode)



public  c_station_ops, c_protect_mode



c_station_ops:          ds      1



    c_protect_mode      bit     c_station_ops.c_protect_mode_bpos

                



rseg                    dcmfw_ext_seg



public  c_save_msg_ptr



c_save_msg_ptr:         ds      1



rseg            dcmfw_code_seg



$eject

;***************************************************************************

;*

;*     ENTRY:          process_rac_cmd

;*

;*     PURPOSE:        handle remote access and control commands

;*

;*     CALLED FROM:    None

;*

;*     ENTERED FROM:   send_local

;*

;*     CALLS:          None

;*

;*     ENTERS:         local_gateway_switch    (msg_ptr in R0)

;*                     rqsystemstart:          reset

;*

;*     INPUTS:         message pointer in R0

;*                     C_Protect_Mode

;*

;*     RETURNS:        None, jump to Local_Gateway_Switch, or RAC procedure

;*

;*     DESCRIPTION:    Procedure begins by setting the Message Type bit to

;*                     a Reply for return.  Then the Message Command is

;*                     checked to see if it is a supported code.  If not,

;*                     the message is returned, via send_local with an

;*                     error.  Otherwise, C_Protect_Mode is checked to see

;*                     if it has been set to stop access types of commands.

;*                     If so, the Message Command is checked to see if

;*                     it falls within the access category.  If it does,

;*                     the message is returned.

;*

;*                     Otherwise, the number of data (non-header) bytes

;*                     in the message is computed.  If the result is

;*                     zero, the message is returned.  Otherwise, result is

;*                     put in R7, the result divided by 2 is put in R6,

;*                     and a pointer to the start of the data

;*                     area is put in R1.  Then a jump is computed to the

;*                     appropriate handler for the command and executed.

;*

;***************************************************************************



c_jmp_base:

    ajmp    c_reset_device                  ;rac command 0

    ajmp    c_create_tsk                    ;rac command 1

    ajmp    c_delete_tsk                    ;rac command 2

    ajmp    c_get_fn_id                     ;rac command 3

    ajmp    c_protect_rac                   ;rac command 4

    ajmp    c_ext_read_io                   ;rac command 5

    ajmp    c_ext_write_io                  ;rac command 6

    ajmp    c_ext_update_io                 ;rac command 7

    ajmp    c_upload_data                   ;rac command 8

    ajmp    c_dnload_data                   ;rac command 9

    ajmp    c_ext_OR_io                     ;rac command A

    ajmp    c_ext_AND_io                    ;rac command B

    ajmp    c_ext_XOR_io                    ;rac command C

    ajmp    c_int_write                     ;rac command D

    ajmp    c_int_read                      ;rac command E

    ajmp    c_node_info                     ;rac command F

    ajmp    c_offline                       ;rac command 10

    ajmp    c_upload_code                   ;rac command 11

    ajmp    c_dnload_code                   ;rac command 12



process_rac_cmd:

    %set_msg_ptr(message_route_ofs)         ;set message type to reply

    mov     a,#reply_msk                    ;

    orl     a,@r1                           ;

    mov     @r1,a                           ;



    %set_msg_ptr(message_command_ofs)       ;check for valid rac

    mov     b,#e_unknown_rac_command        ; preset reply status for error

    %jlt(#max_rac_command_val,@r1,c_error_return)



; Look at the number of bytes of data in the message.  If there are none,

; do nothing, else record the data field length in r7

;

    mov     b,r1                            ;Save Command Address

    %set_msg_ptr(message_length_ofs)        ;

    mov     a,@r1                           ;a = message length

    mov     r1,b                            ;Restore Command Address

    mov     b,#e_service_ok                 ;preset reply for SERVICE$OK

    %jle(A,#message_header_size,c_error_return) ;if no data area, return

    mov     r7,a                            ;r7 = data area length



; For i/o commands, cache the number of accesses (data bytes/2) in r6

; The size of each i/o access is 2.

;

    clr     c                                ;

    rrc     a                                ;

    mov     r6,a                             ;



; Vector to the code to process the command number in @r1

;       

    mov     b,@r1                            ;save command in b

    mov     @r1,#e_service_ok                ;preset response

    %set_msg_ptr(message_data_ofs)           ;r1 = data area base addr

    mov     a,b                              ;a = command

    rl      a                                ;a = offset into jump table

    mov     dptr,#c_jmp_base                 ;dptr => jump table base

    jmp     @a+dptr                          ;jump to appropriate jump vector



c_error_return:

; Error return routine

; Inputs: b = msg. reply status

;         r1 => message.command/reply

;

    mov     a,b                             ;place error in msg. reply field

    mov     @r1,a                           ;

    ajmp    local_gateway_switch            ;and return msg. to sender



;---------------------------------------------------------------------------

; 

;   Entry Point:   ext_read_io

; 

;   Purpose:        read bytes of external memory and return message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R6

; 

;   Returns:        None

; 

;   Description:    Computes number of read requests, reads the

;                   specified external memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_ext_read_io:

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    mov     a,r6                            ;if no data to read, exit

    jz      c_ext_read_io_exit              ;

    mov     dph,#ext_io_page                ;dph => i/o area



c_ext_read_io_loop:

    mov     dpl,@r1                         ;dptr => abs. i/o address

    inc     r1                              ;read i/o byte into message

    movx    a,@dptr                         ;

    mov     @r1,a                           ;

    inc     r1                              ;r1 => next i/o offset addr.

    djnz    r6,c_ext_read_io_loop           ;check for more itemsx



c_ext_read_io_exit:

    ajmp    local_gateway_switch            ;return message



;---------------------------------------------------------------------------

; 

;   Entry Point:   ext_write_io

; 

;   Purpose:        write bytes of external memory and return message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R6

; 

;   Returns:        None

; 

;   Description:    Computes number of write requests, writes the

;                   specified external memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_ext_write_io:

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    mov     a,r6                            ;if no data to write, exit

    jz      c_ext_write_io_exit             ;

    mov     dph,#ext_io_page                ;dph => i/o area



c_ext_write_io_loop:

    mov     dpl,@r1                         ;dptr => abs. i/o address

    inc     r1                              ;write next i/o byte from msg.

    mov     a,@r1                           ;

    movx    @dptr,a                         ;

    inc     r1                              ;r1 => next i/o offset addr.

    djnz    r6,c_ext_write_io_loop          ;check for more items



c_ext_write_io_exit:

    ajmp    local_gateway_switch            ;return message



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   ext_update_io

; 

;   Purpose:        update bytes of external memory and return message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R6

; 

;   Returns:        None

; 

;   Description:    Computes number of update requests, updates the

;                   specified external memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_ext_update_io:

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    mov     a,r6                            ;if no data to update, exit

    jz      c_ext_update_io_exit            ;

    mov     dph,#ext_io_page                ;dph => i/o area



c_ext_update_io_loop:

    mov     dpl,@r1                         ;dptr => abs. i/o address

    inc     r1                              ;write next i/o byte from msg.

    mov     a,@r1                           ;

    movx    @dptr,a                         ;

    movx    a,@dptr                         ;read back i/o byte into msg.

    mov     @r1,a                           ;

    inc     r1                              ;r1 => next i/o offset addr.

    djnz    r6,c_ext_update_io_loop         ;check for more items



c_ext_update_io_exit:

    ajmp    local_gateway_switch            ;return message



;---------------------------------------------------------------------------

; 

;   Entries:        ext_or_io, ext_and_io, and ext_xor_io

; 

;   Purpose:        Perform logical operation on bytes of i/o memory

;                   and return message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R6

; 

;   Returns:        None

; 

;   Description:    Computes number of or, and, or xor requests, reads the

;                   specified external memory locations, performs a logical

;                   op on the value with the message data, writes

;                   to the specified external memory location, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



%*define(ext_logical_io (op)) local log_op_loop log_op_exit

(

%'

%' This macro is used for or-ing, and-ing, and xor-ing bytes of data

%' into i/o ports.

%' Input: %op = or | anl | xrl 

%'

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    mov     a,r6                            ;if no data to update, exit

    jz      %log_op_exit                    ;

    mov     dph,#ext_io_page                ;dph => i/o area



%log_op_loop:

    mov     dpl,@r1                         ;dptr => abs. i/o address

    inc     r1                              ;write next i/o byte from msg.

    movx    a,@dptr                         ;read i/o byte

    %op     a,@r1                           ;perform logical operation on i/o

    movx    @dptr,a                         ;

    movx    a,@dptr                         ;read back byte from port

    mov     @r1,a                           ;and place in message

    inc     r1                              ;r1 => next i/o offset addr.

    djnz    r6,%log_op_loop                 ;check for more items



%log_op_exit:

    ajmp    local_gateway_switch            ;return message

)



c_ext_or_io:

    %ext_logical_io(orl)                    ;perform or-io rac function



c_ext_and_io:

    %ext_logical_io(anl)                    ;perform and-io rac function



c_ext_xor_io:

    %ext_logical_io(xrl)                    ;perform xor-io rac function



p_error_return:

; Rac-Protect error return routine

; Inputs: r1 => message.data

;

    dec     r1                              ;r1 => message.reply

    mov     b,#e_rac_protected              ;

    ajmp    c_error_return                  ;



;---------------------------------------------------------------------------

; 

;   Entry Point:   int_read

; 

;   Purpose:        read bytes of internal memory and return message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R6

; 

;   Returns:        None

; 

;   Description:    Computes number of read requests, reads the

;                   specified internal memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_int_read:

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    mov     a,r6                            ;if no bytes to read, exit

    jz      c_int_read_exit                 ;

    mov     b,r0                            ;save msg ptr in b



c_int_read_repeat:

    mov     ar0,@r1                         ;r0 => byte to be read

    mov     a,@r0                           ;a =  read byte

    inc     r1                              ;r1 => next msg. data space

    mov     @r1,a                           ;write data into msg. data space

    inc     r1                              ;r1 => next msg. addr. value

    djnz    r6,c_int_read_repeat            ;check for more items



    mov     r0,b                            ;restore msg ptr to r0

c_int_read_exit:

    ajmp    local_gateway_switch            ;return message to sender



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   int_write

; 

;   Purpose:        write bytes of internal memory and return message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R6

; 

;   Returns:        None

; 

;   Description:    Computes number of write requests, writes the

;                   specified internal memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_int_write:

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    mov     a,r6                            ;if no bytes to write, exit

    jz      c_int_write_exit                ;

    mov     b,r0                            ;save msg ptr in b



c_int_write_repeat:

    mov     ar0,@r1                         ;r0 => address to be written to

    inc     r1                              ;r1 => next value to be written

    mov     a,@r1                           ;a = value to be written

    mov     @r0,a                           ;write value to int. mem.

    inc     r1                              ;r1 => next msg. addr. value

    djnz    r6,c_int_write_repeat           ;check for more items



    mov     r0,b                            ;restore msg ptr to r0

c_int_write_exit:

    ajmp    local_gateway_switch            ;return message to sender



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   ext_upload_data, ext_upload_code

; 

;   Purpose:        ext_upload_data uploads bytes of external data memory

;                   and returns the message.

;                   ext_upload_code uploads bytes of code memory and returns

;                   the message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R7

; 

;   Returns:        None

; 

;   Description:    Computes number of upload requests, reads the

;                   specified external memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_upload_data:

    clr     b.0                             ;indicate upload ext mem op

    sjmp    c_upload                        ;go upload



c_upload_code:

    setb    b.0                             ;indicate upload code mem op

;   sjmp    c_upload                        ;go upload



c_upload:  

    jb      c_protect_mode,p_error_return   ;if rac protect is in effect

    %jlt(r7,#3,c_ext_upload_exit)           ;if no data to upload, exit

    dec     r7                              ;compute # of iterations

    dec     r7                              ;r7 = # of iterations

    mov     dph,@r1                         ;Set upload start location

    inc     r1                              ;

    mov     dpl,@r1                         ;

        

c_ext_upload_repeat:

    inc     r1                              ;r1 => next data entry

    jb      b.0,move_in_code                ;if uploading code, jump



    movx    a,@dptr                         ;move in next ext data byte

    sjmp    upload_mdata                    ;



move_in_code:

    clr     a                               ;move in next code byte

    movc    a,@a+dptr                       ;



upload_mdata:

    mov     @r1,a                           ;upload data to message  

    inc     dptr                            ;increment to next address

    djnz    r7,c_ext_upload_repeat          ;check for more items



c_ext_upload_exit:

    ajmp    local_gateway_switch            ;return message



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   ext_dnload_data, ext_dnload_code

; 

;   Purpose:        dnload bytes of external memory and return message

;                   ext_dnload_code adds a ~20msec delay between 

;                   byte writes.  This is for downloading code into e2prom.

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R7

; 

;   Returns:        None

; 

;   Description:    Computes number of dnload requests, writes the

;                   specified external memory locations, and 

;                   returns the results in the reply message.

; 

;---------------------------------------------------------------------------



c_dnload_data:

    clr     b.0                             ;clear code flag

    sjmp    c_dnload                        ;go execute dnload routine



c_dnload_code:

    setb    b.0                             ;set code flag

;   sjmp    c_dnload                        ;go execute dnload routine



c_dnload:    

    jb      c_protect_mode,p_error_return   ;if rac protect off, jump

    %jlt(r7,#3,c_ext_dnload_exit)           ;if no data to dnload, exit

    dec     r7                              ;

    dec     r7                              ;r7 = # of dnload iterations

    mov     dph,@r1                         ;set external data location

    inc     r1                              ;

    mov     dpl,@r1                         ;



c_ext_dnload_repeat:

    inc     r1                              ;move to next data entry

    mov     a,@r1                           ;dnload message data

    movx    @dptr,a                         ;dnload data

    jnb     b.0,skip_delay                  ;if dnloading data, skip delay



    mov     r6,#100                         ;delay ~20 ms before writing

loop1:                                      ;

    mov     a,#100                          ;

loop2:                                      ;

    djnz    acc,loop2                       ;

    djnz    r6,loop1                        ;

skip_delay:



    inc     dptr                            ;increment to next address,

    djnz    r7,c_ext_dnload_repeat          ;check for more items



c_ext_dnload_exit:

    ajmp    local_gateway_switch            ;return message



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   c_create_task

; 

;   Purpose:        execute create task from message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          rqcreatetask

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

; 

;   Returns:        None

; 

;   Description:    Gets the ITD from the message and calls rqcreatetask.

;                   The status return from create task is set as the

;                   message status, and the reply message is returned.

; 

;---------------------------------------------------------------------------



c_create_tsk:

    mov     dptr,#c_save_msg_ptr            ;save msg. ptr.

    mov     a,r0                            ;

    movx    @dptr,a                         ;

    mov     a,@r1                           ;point dptr to itd

    mov     dph,a                           ;

    inc     r1                              ;

    mov     a,@r1                           ;

    mov     dpl,a                           ;dptr => itd

    call    reqcreatetask                   ;create the task

    mov     b,a                             ;b = createtask status

    mov     dptr,#c_save_msg_ptr            ;restore msg. pointer

    movx    a,@dptr                         ;

    mov     r0,a                            ;

    add     a,#message_command_ofs          ;write createtask status to msg.

    mov     r1,a                            ;

    mov     @r1,b                           ;

    ajmp    local_gateway_switch            ;return message



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   c_delete_task

; 

;   Purpose:        execute delete task from message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          rqdeletetask

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

; 

;   Returns:        None

; 

;   Description:    Gets the task_id from the message and calls rqdeletetask.

;                   The status return from delete task is set as the

;                   message status, and the reply message is returned.

; 

;---------------------------------------------------------------------------



c_delete_tsk:

    mov     dptr,#c_save_msg_ptr            ;save msg. ptr.

    mov     a,r0                            ;

    movx    @dptr,a                         ;

    mov     a,@r1                           ;a = task id

    call    reqdeletetask                   ;delete the task

    mov     b,a                             ;b = deletetask status

    mov     dptr,#c_save_msg_ptr            ;restore msg. pointer

    movx    a,@dptr                         ;

    mov     r0,a                            ;

    add     a,#message_command_ofs          ;write deletetask status to msg.

    mov     r1,a                            ;

    mov     @r1,b                           ;

    ajmp    local_gateway_switch            ;return message



;---------------------------------------------------------------------------

; 

;   Entry Point:   c_get_function_ids

; 

;   Purpose:        execute get_function_ids from message

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          rqgetfunctionids

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

; 

;   Returns:        None

; 

;   Description:    Sets the function_block from the message and calls

;                   rqgetfunctionids and the reply message is returned.

; 

;---------------------------------------------------------------------------



c_get_fn_id:

    mov     dptr,#c_save_msg_ptr            ;save msg. ptr.

    mov     a,r0                            ;

    movx    @dptr,a                         ;

    mov     dph,#0                          ;point dptr to msg.data

    mov     dpl,r1                          ;

    call    reqgetfunctionids               ;get function ids

    mov     dptr,#c_save_msg_ptr            ;restore msg. pointer

    movx    a,@dptr                         ;

    mov     r0,a                            ;

    add     a,#message_command_ofs          ;write status into msg.reply

    mov     r1,a                            ;

    mov     @r1,#0                          ;

    ajmp    local_gateway_switch            ;return message

 

;---------------------------------------------------------------------------

;

;   Entry Point:    c_protect_rac

; 

;   Purpose:        Turn off access capability of rac task

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

; 

;   Returns:        None

; 

;   Description:    Sets or clears the c_protect_mode flag and 

;                   the reply message is returned.

; 

;---------------------------------------------------------------------------



c_protect_rac:

    cjne    @r1,#0,c_rac_on                 ;if protect value <> 0, jump

    clr     c_protect_mode                  ;value=0, so set protect mode off

    ajmp    local_gateway_switch            ;and return msg. to sender



c_rac_on:

    setb    c_protect_mode                  ;value=1, so set protect mode on

    ajmp    local_gateway_switch            ;and return msg. to sender



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   c_reset_device

; 

;   Purpose:        Reset device by jumping to reset vector location

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         rqsystemstart

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

; 

;   Returns:        None

; 

;   Description:    Jumps to rqsystemstart, the reset vector

; 

;---------------------------------------------------------------------------



c_reset_device:

    ljmp    rqsystemstart    



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   node_info  

; 

;   Purpose:        Provide node identification information.            

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R7

; 

;   Returns:        None

; 

;   Description:    Move the node name, version, mode byte, and buffer

;                   size into the data area, and return the message.

; 

;---------------------------------------------------------------------------



node_table:        DB    'i8044 21',0,0,0,0,0

node_table_length  EQU   $-node_table



c_node_info:

    mov   dptr,#node_table                  ;move in node name (='i8044')

    mov   r6,#node_table_length             ;and fw_version (='21')

c_node_loop:                                ;

    clr   a                                 ;

    movc  a,@a+dptr                         ;

    mov   @r1,a                             ;

    inc   dptr                              ;

    inc   r1                                ;

    djnz  r6,c_node_loop                    ;

    mov   a,r1                              ;move configuration byte into m0

    subb  a,#5                              ;                 

    mov   r1,a                              ; r1 => m0

    mov   dptr,#0fffeh                      ; dptr => configuration byte

    movx  a,@dptr                           ;

    mov   @r1,a                             ;

    inc   r1                                ;r1 => m1

    mov   @r1,rqbufsize                     ;move in maximum message size

    ajmp  local_gateway_switch              ;return message



 

;---------------------------------------------------------------------------

; 

;   Entry Point:   offline    

; 

;   Purpose:        Set a slave node to the offline state.              

; 

;   Entered From:   process_rac_cmd

; 

;   Calls:          none

; 

;   Enters:         local_gateway_switch    (msg_ptr in R0)

; 

;   Inputs:         message pointer in R0

;                   data pointer in R1

;                   data count in R7

; 

;   Returns:        None

; 

;   Description:    If this station is not a master, do nothing.  Else, 

;                   set the designated slave offline by setting its

;                   station poll entry to offline, and if this is the

;                   present station (= siu stad), then also set the

;                   siu nsnr byte to offline.

; 

;---------------------------------------------------------------------------



c_offline:

    jnb   s_station_mode,co_pass            ;if i'm not a master, do nothing

    mov   a,@r1                             ; a = slave station addr.

    acall set_slave_offline                 ;else set slave station offline

co_pass:

    ajmp  local_gateway_switch              ;return message



END

